import csv
import datetime
import ladok3
import os
import sys

def filter_rounds(all_rounds, desired_rounds):
  """Returns only the round objects with round code in desired_rounds."""
  if not desired_rounds:
    return all_rounds
  return filter(
    lambda x: x.round_code in desired_rounds,
    all_rounds
  )
def extract_data_for_round(ladok, course_round, args):
  course_start = course_round.start
  course_length = course_round.end - course_start
  component = course_round.components()[0]
  results = ladok.search_reported_results_JSON(
    course_round.round_id, component.instance_id)

  students = filter_students(course_round.participants(), args.students)

  for student in students:
    student_results = filter_student_results(student, results)

    if not should_include(ladok, student, course_round, student_results):
      continue

    components = filter_components(course_round.components(), args.components)

    for component in components:
      if len(student_results) < 1:
        result_data = None
      else:
        result_data = filter_component_result(
          component, student_results[0]["ResultatPaUtbildningar"])

      if not result_data:
        grade = "-"
        normalized_date = None
      else:
        if "Betygsgradsobjekt" in result_data:
          grade = result_data["Betygsgradsobjekt"]["Kod"]
          date = datetime.date.fromisoformat(
            result_data["Examinationsdatum"])
          normalized_date = (date - course_start) / course_length
          if args.time_limit and normalized_date > args.time_limit:
            grade = "-"
            normalized_date = None
        else:
          grade = "-"
          normalized_date = None

      yield student, component, grade, normalized_date
def filter_student_results(student, results):
  return list(filter(
    lambda x: x["Student"]["Uid"] == student.ladok_id,
    results))
def filter_component_result(component, results):
  for component_result in results:
    if "Arbetsunderlag" in component_result:
      result_data = component_result["Arbetsunderlag"]
    elif "SenastAttesteradeResultat" in component_result:
      result_data = component_result["SenastAttesteradeResultat"]
    else:
      continue
    if component.instance_id != result_data["UtbildningsinstansUID"]:
      continue
    return result_data

  return None
def filter_students(all_students, desired_students):
  """Returns only the students with personnummer in desired_students."""
  if not desired_students:
    return all_students
  return filter(
    lambda x: x.personnummer in desired_students,
    all_students
  )

def filter_components(all_components, desired_components):
  """Returns only the components with a code in the desired_components."""
  if not desired_components:
    return all_components
  return filter(
    lambda x: x.code in desired_components,
    all_components
  )
def should_include(ladok, student, course_round, result):
  """Returns True if student should be included, False if to be excluded"""
  if is_reregistered(ladok, student.ladok_id, course_round):
    return False

  if has_credit_transfer(result):
    return False

  return True
def is_reregistered(ladok, student_id, course):
  """Check if the student is reregistered on the course round course."""
  registrations = ladok.registrations_on_course_JSON(
    course.education_id, student_id)
  registrations.sort(
    key=lambda x: x["Utbildningsinformation"]["Studieperiod"]["Startdatum"])
  first_reg = registrations[0]
  return first_reg["Utbildningsinformation"]["Utbildningstillfalleskod"] != \
    course.round_code
def has_credit_transfer(results):
  """Returns True if there exists a credit tranfer among the results."""
  for result in results:
    for component_result in result["ResultatPaUtbildningar"]:
      if component_result["HarTillgodoraknande"]:
        return True

  return False

def add_command_options(parser):
  data_parser = parser.add_parser("data",
    help="Returns course results data in CSV form",
    description="""
  Returns the results in CSV form for all first-time registered students.
  """.strip())
  data_parser.set_defaults(func=command)
  data_parser.add_argument("course_code",
    help="The course code of the course for which to export data")

  data_parser.add_argument("-d", "--delimiter",
    default="\t",
    help="The delimiter for the CSV output; "
      "default is a tab character to be compatible with POSIX commands, "
      "use `-d,` or `-d ,` to get comma-separated values.")
  data_parser.add_argument("-r", "--rounds", nargs="+",
    help="The round codes for the rounds to include, "
      "otherwise all rounds will be included.")
  data_parser.add_argument("-t", "--time-limit", type=float,
    help="The time (normalized) for cutting off results, "
      "use `-t 1.0` to cut off at course end.")
  data_parser.add_argument("-s", "--students", nargs="+",
    help="List of personnummer for students to include, "
      "otherwise all students will be included.")

  data_parser.add_argument("-c", "--components", nargs="+",
    help="List of component codes for components to include, "
      "otherwise all components will be included.")

def command(ladok, args):
  data_writer = csv.writer(sys.stdout, delimiter=args.delimiter)
  course_rounds = filter_rounds(
    ladok.search_course_rounds(code=args.course_code),
    args.rounds)

  data_writer.writerow([
    "Course", "Round", "Component", "Student", "Grade", "Time"
  ])
  for course_round in course_rounds:
    data = extract_data_for_round(ladok, course_round, args)

    for student, component, grade, time in data:
      data_writer.writerow(
        [course_round.code, course_round.round_code, component,
          student, grade, time]
      )



#!/bin/env python3
"""A command-line interface for LADOK 3"""

import appdirs
import argcomplete, argparse
from cryptography.fernet import Fernet
import getpass
import json
import keyring
import os
import pickle
import re
import sys
import traceback

import ladok3.data
import ladok3.report

dirs = appdirs.AppDirs("ladok", "dbosk@kth.se")

def err(rc, msg):
  print(f"{sys.argv[0]}: error: {msg}", file=sys.stderr)
  traceback.print_exc(file=sys.stderr)
  sys.exit(rc)

def warn(msg):
  print(f"{sys.argv[0]}: {msg}", file=sys.stderr)
def store_ladok_session(ls):
  if not os.path.isdir(dirs.user_cache_dir):
    os.mkdir(dirs.user_cache_dir)

  file_path = dirs.user_cache_dir + "/LadokSession"

  pickled_ls = pickle.dumps(ls)
  key = keyring.get_password("ladok3", "cache_key")
  if not key:
    key = Fernet.generate_key()
  keyring.set_password("ladok3", "cache_key", key)

  fernet_protocol = Fernet(key)
  encrypted_ls = fernet_protocol.encrypt(pickled_ls)

  with open(file_path, "wb") as file:
    file.write(encrypted_ls)
  
def restore_ladok_session():
  file_path = dirs.user_cache_dir + "/LadokSession"

  if os.path.isfile(file_path):
    with open(file_path, "rb") as file:
      encrypted_ls = file.read()
      key = keyring.get_password("ladok3", "cache_key")

      fernet_protocol = Fernet(key)
      pickled_ls = fernet_protocol.decrypt(encrypted_ls)
      return pickle.loads(pickled_ls)

  return None
def update_credentials_in_keyring(ls, args):
  user = input("LADOK username: ")
  passwd = getpass.getpass("LADOK password: ")

  keyring.set_password("ladok3", "username", user)
  keyring.set_password("ladok3", "password", passwd)

  clear_cache(ls, args)
def load_credentials(filename="config.json"):
  """Load credentials from environment or file named filename"""
  try:
    user = keyring.get_password("ladok3", "username")
    passwd = keyring.get_password("ladok3", "password")
    if user and passwd:
      return user, passwd
  except:
    pass

  try:
    user = os.environ["LADOK_USER"]
    passwd = os.environ["LADOK_PASS"]
    return user, passwd
  except:
    pass

  try:
    with open(filename) as conf_file:
      config = json.load(conf_file)
    return config["username"], config["password"]
  except:
    pass

  return None, None
def clear_cache(ls, args):
  try:
    os.remove(dirs.user_cache_dir + "/LadokSession")
  except FileNotFoundError as err:
    pass

  sys.exit(0)
def main():
  """Run the command-line interface for the ladok command"""
  argp = argparse.ArgumentParser(
    description="This is a CLI-ification of LADOK3's web GUI.",
    epilog="Web: https://github.com/dbosk/ladok3"
  )
  argp.add_argument("-f", "--config-file",
    default=f"{dirs.user_config_dir}/config.json",
    help="Path to configuration file "
      f"(default: {dirs.user_config_dir}/config.json) "
      "or set LADOK_USER and LADOK_PASS environment variables.")
  subp = argp.add_subparsers(
    title="commands",
    dest="command",
    required=True
  )
  login_parser = subp.add_parser("login",
    help="Manage login credentials",
    description="""
  Manages the user's LADOK login credentials. There are three ways to supply the 
  login credentials, in order of priority:

  1) Through the system keyring: Just run `ladok login` and you'll be asked to 
     enter the credentials and they will be stored in the keyring.

  2) Through the environment: Just set the environment variables LADOK_USER and 
     LADOK_PASS.

  3) Through the configuration file: Just write

        {
          "username": "the actual username",
          "password": "the actual password"
        }

     to the file """ + dirs.user_config_dir + """/config.json (default, or use 
     the -f option).
  """
  )
  login_parser.set_defaults(func=update_credentials_in_keyring)
  cache_parser = subp.add_parser("cache",
    help="Manage cache",
    description="Manages the cache of LADOK data"
  )
  cache_subp = cache_parser.add_subparsers(
    title="subcommands",
    dest="subcommand",
    required=True
  )
  cache_clear = cache_subp.add_parser("clear",
    help="Clear the cache",
    description="Clears everything from the cache"
  )
  cache_clear.set_defaults(func=clear_cache)
  ladok3.data.add_command_options(subp)
  ladok3.report.add_command_options(subp)
  argcomplete.autocomplete(argp)
  args = argp.parse_args()
  ls = restore_ladok_session()
  if not ls:
    LADOK_USER, LADOK_PASS = load_credentials(args.config_file)
    ls = ladok3.LadokSessionKTH(LADOK_USER, LADOK_PASS)
  if "func" in args:
    args.func(ls, args)
  store_ladok_session(ls)

if __name__ == "__main__":
  try:
    main()
    sys.exit(0)
  except Exception as e:
    err(-1, e)



import csv
import datetime
import ladok3
import sys

def report_one_result(ladok, args):
  if not (args.course_code and args.component_code and
    args.student_id and args.grade):
    print(f"{sys.argv[0]} report: "
      "not all positional args given: course_code, component, student, grade",
      file=sys.stderr)
    sys.exit(1)
  try:
    student = ladok.get_student(args.student_id)
    course = student.courses(code=args.course_code)[0]
    result = course.results(component=args.component_code)[0]
    result.set_grade(args.grade, args.date)
    if args.finalize:
      result.finalize()
  except Exception as err:
    try:
      print(f"{student}: {err}")
    except ValueError as verr:
      print(f"{verr}: {args.student_id}: {err}")

def report_many_results(ladok, args):
  data_reader = csv.reader(sys.stdin, delimiter=args.delimiter)
  for course_code, component_code, student_id, grade, date in data_reader:
    try:
      student = ladok.get_student(student_id)
      course = student.courses(code=course_code)[0]
      result = course.results(component=component_code)[0]
      result.set_grade(grade, date)
      if args.finalize:
        result.finalize()
    except Exception as err:
      try:
        print(f"{student}: {err}")
      except ValueError as verr:
        print(f"{verr}: {args.student_id}: {err}")

def add_command_options(parser):
  report_parser = parser.add_parser("report",
    help="Reports course results to LADOK",
    description="Reports course results to LADOK"
  )
  report_parser.set_defaults(func=command)
  one_parser = report_parser.add_argument_group(
    "one result as positional args, only date is optional")
  one_parser.add_argument("course_code", nargs="?",
    help="The course code (e.g. DD1315) for which the grade is for"
  )

  one_parser.add_argument("component_code", nargs="?",
    help="The component code (e.g. LAB1) for which the grade is for"
  )

  one_parser.add_argument("student_id", nargs="?",
    help="Student identifier (personnummer or LADOK ID)"
  )

  one_parser.add_argument("grade", nargs="?",
    help="The grade (e.g. A or P)"
  )
  one_parser.add_argument("date", nargs="?",
    help="Date on ISO format (e.g. 2021-03-18), "
      f"defaults to today's date ({datetime.date.today()})",
    type=datetime.date.fromisoformat,
    default=datetime.date.today()
  )
  many_parser = report_parser.add_argument_group(
    "many results read from standard input as CSV, columns: "
    "course, component, student, grade, date.")
  many_parser.add_argument("-d", "--delimiter",
    default="\t",
    help="The delimiter for the CSV input; "
      "default is a tab character to be compatible with POSIX commands, "
      "use `-d,` or `-d ,` to get comma-separated values.")
  report_parser.add_argument("-f", "--finalize",
    help="Finalize the grade (mark as ready/klarmarkera) for examiner to attest",
    action="store_true",
    default=False
  )

def command(ladok, args):
  if args.course_code:
    report_one_result(ladok, args)
  else:
    report_many_results(ladok, args)
